"use client";
import { useState, useEffect } from "react";
import PageHeader from "../../components/page-header";
import StoreCard from "../../components/store-card";
import { Button } from "@/components/ui/button";
import { Star, ShoppingCart } from "lucide-react";
import Image from "next/image";
import { ReviewDes } from "../review-des";
import DownloadAppCard from "@/app/components/DownloadAppCard";
import useSWR from "swr";
import { ToastContainer, toast } from "react-toastify";
import "react-toastify/dist/ReactToastify.css";
import { useCart } from "@/app/context/CartContext";
const fetcher = (url) => fetch(url).then((res) => res.json());

export default function Page({ params }) {
  const { cartCount, setCartCount } = useCart();
  const { id } = params;

  const { data, error } = useSWR(
    `${process.env.NEXT_PUBLIC_BASE_URL}getProductsDetail?product_id=${id}`,
    fetcher
  );

  const product = data?.product;

  if (error) {
    console.error("There was an error fetching the product");
  }

  const [quantity, setQuantity] = useState(1);
  const [selectedParentVariation, setSelectedParentVariation] = useState(null);
  const [selectedChildVariation, setSelectedChildVariation] = useState(null);
  const [selectedImage, setSelectedImage] = useState(null);
  const [availableStock, setAvailableStock] = useState(0);
  const [price, setPrice] = useState(0);
  const [cart, setCart] = useState([]);

  // Load cart from local storage when component mounts
  useEffect(() => {
    const storedCart = localStorage.getItem("cart");
    if (storedCart) {
      setCart(JSON.parse(storedCart));
    }
  }, []);

  // Set default product details when data loads
  useEffect(() => {
    if (product) {
      const defaultParent = product.product_variation?.[0] || null;
      setSelectedParentVariation(defaultParent);
      setPrice(defaultParent?.parentprice || product.product_discounted_price);
      setAvailableStock(defaultParent?.parentstock || 0);
      setSelectedImage(
        defaultParent?.parentimage || product.product_images?.[0]
      );

      if (defaultParent?.children?.length) {
        setSelectedChildVariation(defaultParent.children[0]);
        setPrice(defaultParent.children[0]?.price);
        setAvailableStock(defaultParent.children[0]?.stock);
        setSelectedImage(defaultParent.children[0]?.image);
      }
    }
  }, [product]);

  // Handle parent variation change
  const handleParentVariationChange = (parentVariation) => {
    setSelectedParentVariation(parentVariation);
    setSelectedChildVariation(null);
    setSelectedImage(parentVariation.parentimage);
    setPrice(parentVariation.parentprice);
    setAvailableStock(parentVariation.parentstock);

    if (parentVariation.children?.length) {
      setSelectedChildVariation(parentVariation.children[0]);
      setPrice(parentVariation.children[0]?.price);
      setAvailableStock(parentVariation.children[0]?.stock);
      setSelectedImage(parentVariation.children[0]?.image);
    }
  };

  // Handle child variation change
  const handleChildVariationChange = (childVariation) => {
    setSelectedChildVariation(childVariation);
    setSelectedImage(childVariation.image);
    setPrice(childVariation.price);
    setAvailableStock(childVariation.stock);
  };

  // Compare variations objects for cart duplicate check
  const areVariationsEqual = (var1, var2) => {
    const keys1 = Object.keys(var1);
    const keys2 = Object.keys(var2);
    if (keys1.length !== keys2.length) return false;
    return keys1.every((key) => var1[key] === var2[key]);
  };

  // Add to Cart function
  const addToCart = () => {
    if (product.product_variation.length > 0 && !selectedParentVariation) {
      toast.error("Please select a variation.", { autoClose: 800 });
      return;
    }
    if (
      selectedParentVariation?.children?.length > 0 &&
      !selectedChildVariation
    ) {
      toast.error(
        `Please select a ${selectedParentVariation.children[0]?.child_option_name} option.`,
        { autoClose: 800 }
      );

      return;
    }

    // Flatten variations directly into the item
    const variations = selectedParentVariation
      ? {
          [selectedParentVariation.parent_option_name]:
            selectedParentVariation.parentname,
          ...(selectedChildVariation && {
            [selectedChildVariation.child_option_name]:
              selectedChildVariation.name,
          }),
        }
      : {};

    // Construct the full image URL
    const imageUrl = selectedImage
      ? `${process.env.NEXT_PUBLIC_IMAGE_URL}${selectedImage}`
      : `${process.env.NEXT_PUBLIC_IMAGE_URL}${product.product_images?.[0]}`;

    // Construct the cart item without a nested variations object
    const item = {
      product_id: product.product_id,
      quantity,
      price,
      image: imageUrl, // Add the image URL to the cart item
      ...variations, // Spread variations directly into the item
    };

    // Check if item already exists in cart
    const existingItemIndex = cart.findIndex(
      (cartItem) =>
        cartItem.product_id === item.product_id &&
        areVariationsEqual(
          { Color: cartItem.Color, Memory: cartItem.Memory },
          { Color: item.Color, Memory: item.Memory }
        )
    );

    let updatedCart;
    if (existingItemIndex >= 0) {
      // Update quantity if item exists
      updatedCart = [...cart];
      updatedCart[existingItemIndex].quantity += quantity;
    } else {
      // Add new item
      updatedCart = [...cart, item];
    }

    // Update state and local storage
    setCartCount(updatedCart);
    setCart(updatedCart);
    localStorage.setItem("cart", JSON.stringify(updatedCart));
    toast.success("Item added to cart!", { autoClose: 500 });
  };

  if (!data) return <LoadingSkeleton />;

  function LoadingSkeleton() {
    return (
      <>
        <PageHeader title="Shop" breadcrumb="Home - Product" />
        <div className="mx-auto w-full p-4 max-w-6xl animate-pulse">
          <div className="flex flex-col md:flex-row gap-6">
            <div className="w-full md:w-[40%]">
              <div className="w-full h-[300px] bg-gray-200 rounded-lg" />
              <div className="flex gap-4 mt-2">
                {[...Array(4)].map((_, i) => (
                  <div key={i} className="h-16 w-16 bg-gray-200 rounded-lg" />
                ))}
              </div>
            </div>
            <div className="w-full md:w-[45%] space-y-4">
              <div className="h-6 w-3/4 bg-gray-200 rounded" />
              <div className="h-4 w-full bg-gray-200 rounded" />
              <div className="h-4 w-2/3 bg-gray-200 rounded" />
              <div className="h-10 w-1/4 bg-gray-200 rounded" />
              <div className="h-10 w-1/4 bg-gray-200 rounded" />
            </div>
          </div>
        </div>
      </>
    );
  }

  return (
    <>
      <PageHeader title="Shop" breadcrumb="Home - Product" />
      <div className="mx-auto w-full p-4 max-w-6xl">
        <div className="flex flex-col md:flex-row gap-6">
          {/* Section Images */}
          <div className="w-full md:w-[40%]">
            <div className="relative w-full">
              <Image
                src={`${process.env.NEXT_PUBLIC_IMAGE_URL}${selectedImage}`}
                alt="Product"
                width={400}
                height={200}
                // unoptimized
                className="rounded-lg object-contain w-full"
              />
            </div>
            <div className="flex gap-4 mt-2 overflow-x-auto">
              {product?.product_images?.map((image, index) => (
                <Image
                  key={index}
                  className={`object-cover h-16 w-16 cursor-pointer border-2 rounded-lg ${
                    selectedImage === image
                      ? "border-primary"
                      : "border-transparent"
                  }`}
                  src={`${process.env.NEXT_PUBLIC_IMAGE_URL}${image}`}
                  alt={`Product image ${index + 1}`}
                  width={100}
                  height={100}
                  onClick={() => setSelectedImage(image)}
                />
              ))}
            </div>
          </div>
          {/* Product Details */}
          <div className="w-full md:w-[45%]">
            <h1 className="text-xl">{product?.product_name}</h1>
            <p className="text-customGrayBlack text-sm mt-2">
              {product?.product_description}
            </p>
            <div className="flex items-center space-x-2 mt-2">
              <Star size={15} className="text-yellow-500 fill-current" />
              <Star size={15} className="text-yellow-500 fill-current" />
              <Star size={15} className="text-yellow-500 fill-current" />
              <Star size={15} className="text-yellow-500 fill-current" />
              <span className="text-custom15 text-customGrayBlack">
                ({product?.review_count} Reviews)
              </span>
            </div>
            <h2 className="text-customGrayBlack text-custom16 mt-3">
              Brand:{" "}
              <span className="text-black font-medium ml1">
                {product?.product_brand}
              </span>
            </h2>
            <div className="mt-4">
              <span className="text-lg font-bold text-red-500">
                Rs. {price}
              </span>
            </div>
            <div className="mt-1">
              <span className="text-gray-500 line-through">
                Rs. {product?.product_price}
              </span>
              <span className="text-black ml-2">
                -{product?.product_discount}%
              </span>
            </div>
            {/* Parent Variation Selection */}
            {product?.product_variation?.length > 0 && (
              <div className="mt-5 flex gap-4">
                <h3 className="text-custom16 font-custom500 w-28">
                  {product.product_variation[0]?.parent_option_name ||
                    "Variation"}
                  :
                </h3>
                <div className="">
                  {product.product_variation.map((parentVariation) => (
                    <button
                      key={parentVariation.parentname}
                      className={`text-customGrayBlack text-xs px-2 py-1 border rounded-sm my-1.5 mx-1.5  ${
                        selectedParentVariation?.parentname ===
                        parentVariation.parentname
                          ? "border-primary"
                          : "border-grayBlack"
                      }`}
                      onClick={() =>
                        handleParentVariationChange(parentVariation)
                      }
                    >
                      {parentVariation.parentname}
                    </button>
                  ))}
                </div>
              </div>
            )}
            {/* Child Variation Selection */}
            {selectedParentVariation?.children?.length > 0 && (
              <div className="mt-5 flex gap-4">
                <h3 className="text-custom16 font-custom500 w-28">
                  {selectedParentVariation.children[0]?.child_option_name ||
                    "Sub-Variation"}
                  :
                </h3>
                <div>
                  {selectedParentVariation.children.map((childVariation) => (
                    <button
                      key={childVariation.name}
                      className={`text-customGrayBlack text-xs px-2 py-1 border rounded-sm my-1.5 mx-1.5 ${
                        selectedChildVariation?.name === childVariation.name
                          ? "border-primary"
                          : "border-grayBlack"
                      }`}
                      onClick={() => handleChildVariationChange(childVariation)}
                    >
                      {childVariation.name}
                    </button>
                  ))}
                </div>
              </div>
            )}
            {/* Quantity */}
            <div className="mt-5 flex gap-4">
              <h3 className="text-custom16 font-custom500 w-28">Quantity:</h3>
              <div className="space-x-3">
                <Button
                  className="text-customGrayBlack text-xs px-2 h-6 border border-grayBlack rounded-sm"
                  variant="outline"
                  onClick={() => setQuantity((prev) => Math.max(1, prev - 1))}
                >
                  -
                </Button>
                <span className="text-xs font-semibold">{quantity}</span>
                <Button
                  className="text-customGrayBlack text-xs px-2 h-6 border border-grayBlack rounded-sm"
                  variant="outline"
                  onClick={() =>
                    setQuantity((prev) =>
                      prev < availableStock ? prev + 1 : prev
                    )
                  }
                >
                  +
                </Button>
              </div>
            </div>
            {/* Add to Cart Button */}
            <div className="mt-10 gap-6 flex">
              <button
                className="bg-primary text-sm py-3 px-2 text-white font-semibold rounded-sm flex gap-1 w-full justify-center"
                onClick={addToCart}
              >
                <ShoppingCart size={16} /> Add to Cart
              </button>
            </div>
          </div>
          <div className="w-full hidden md:block md:w-[14%] flex justify-center md:justify-end">
            <Image
              className="w-full rounded-md object-contain"
              src="/assets/product-test-img/store.png"
              alt="Store"
              width={250}
              height={400}
            />
          </div>
        </div>
        <div className="mt-5 flex flex-col md:flex-row gap-4">
          <StoreCard
            storeName={product?.store_profile_detail?.store_name}
            storeImage={product?.store_profile_detail?.store_image}
          />
          <DownloadAppCard />
        </div>
        <div className="mt-5 w-full">
          <ReviewDes description={product?.product_description} />
        </div>
        <ToastContainer />
      </div>
    </>
  );
}
